<?php
// $Id: biblio_advanced_import.admin.inc,v 1.1.2.2 2011/02/14 12:46:50 mkalkbrenner Exp $

/**
 * @file
 * Administration interface to configureable a duplicate
 * detection strategy for biblio records
 *
 * @see biblio_advanced_import.module
 * @see biblio.module
 *
 * @author Markus Kalkbrenner | Cocomore AG
 *   @see http://drupal.org/user/124705
 */

/**
 * Provides biblio_advanced_import_settings_form
 * using system_settings_form().
 */
function biblio_advanced_import_settings_form() {
  $options = array(
    'create duplicate' => t('Create duplicate record.'),
    'skip import' => t('Skip import of a record.'),
    'update latest' => t('Update the latest existing duplicate record.'),
    'update oldest' => t('Update the oldest existing duplicate record.'),
    'update all' => t('Update all existing duplicate records.'),
    'new rev latest' => t('Create a new revision of the latest existing duplicate record.'),
    'new rev oldest' => t('Create a new revision of the oldest existing duplicate record.'),
    'new rev all' => t('Create a new revisions of all existing duplicate records.'),
  );

  $form['biblio_advanced_import_duplicate_strategy'] = array(
    '#title' => t('Duplicate Strategy'),
    '#description' => t('Create a duplicate or skip a single record of an import or update existing records, if the import detects an already exiting biblio record.'),
    '#type' => 'radios',
    '#options' => $options,
    '#default_value' => variable_get('biblio_advanced_import_duplicate_strategy', 'create duplicate'),
  );

  $options = array(
    'override' => t('Override the existing record.'),
    'override but keep additional' => t('Override the existing record but keep non empty values that do not exist in the record to be imported.'),
    'add new' => t('Only add new values that are empty in the existing record.'),
    'override existing non empty' => t('Only override non empty existing values.'),
    'override existing non empty with non empty' => t('Only override non empty existing values with new non empty existing values.'),
  );

  $form['biblio_advanced_import_merge_strategy'] = array(
    '#title' => t('Merge Strategy'),
    '#description' => t('If you did not choose "Create duplicate record." or "Skip import of a record." as Duplicate Strategy, you have to select a Merge Strategy as well.'),
    '#type' => 'radios',
    '#options' => $options,
    '#default_value' => variable_get('biblio_advanced_import_merge_strategy', 'override'),
  );

  $form['biblio_advanced_import_duplicate_criteria_fieldset'] = array(
    '#title' => t('Duplicate Criteria'),
    '#description' => t('Select the criteria to detect duplicate biblio records. But think twice about your selection. P.e. the Pubmed ID is a perfect criteria. But if you import the same record from two different sources an one contains the Pubmed ID while the other does not, they will not be considered as duplicates.'),
    '#type' => 'fieldset',
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );

  $options = array('title' => 'title');
  $result = db_query("SELECT name FROM {biblio_fields}");
  while ($row = db_fetch_object($result)) {
    $options[$row->name] = $row->name;
  }

  $form['biblio_advanced_import_duplicate_criteria_fieldset']['biblio_advanced_import_duplicate_criteria'] = array(
    '#type' => 'checkboxes',
    '#options' => $options,
    '#default_value' => variable_get('biblio_advanced_import_duplicate_criteria', array('title' => 'title', 'biblio_year' => 'biblio_year')),
  );

  $form['#submit'] = array('biblio_advanced_import_settings_form_submit');

  return system_settings_form($form);
}


/**
 * Submit handler of biblio_advanced_import_settings_form().
 *
 * Checks if duplicate strategy has been changed which requires
 * an update of all hashes of all existing biblio nodes.
 * If required, a batch process will be started to recalculate
 * all hashes.
 */
function biblio_advanced_import_settings_form_submit($form, &$form_state) {
  if ($form_state['values']['biblio_advanced_import_duplicate_strategy'] != 'create duplicate') {
    $update_hashes = FALSE;
    if (variable_get('biblio_advanced_import_duplicate_strategy', 'create duplicate') == 'create duplicate') {
      $update_hashes = TRUE;
    }
    else {
      $old_values = variable_get('biblio_advanced_import_duplicate_criteria', array('title' => 'title', 'biblio_year' => 'biblio_year'));
      foreach($form_state['values']['biblio_advanced_import_duplicate_criteria'] as $key => $value) {
        if ($value) {
          if (!array_key_exists($key, $old_values) || ((string) $old_values[$key] != (string) $value)) {
            $update_hashes = TRUE;
            break;
          }
        }
      }
    }

    if ($update_hashes) {
      $batch = array(
        'title' => t('Re-hashing existing biblio nodes'),
        'operations' => array(
          array('biblio_advanced_import_update_hashes_batch', array()),
        ),
        'finished' => 'biblio_advanced_import_update_hashes_batch_finished',
        'file' => 'biblio_advanced_import.batch.inc',
      );
      batch_set($batch);

      // TODO block import and creation of new biblio nodes until batch is finished
    }
  }
}
